package envoy.client.ui.controller;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;

import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonType;
import javafx.scene.control.ListView;

import envoy.client.data.Chat;
import envoy.client.data.LocalDB;
import envoy.client.event.SendEvent;
import envoy.client.ui.ClearableTextField;
import envoy.client.ui.SceneContext;
import envoy.client.ui.listcell.ContactListCellFactory;
import envoy.event.ElementOperation;
import envoy.event.EventBus;
import envoy.event.contact.ContactOperation;
import envoy.event.contact.ContactSearchRequest;
import envoy.event.contact.ContactSearchResult;
import envoy.util.EnvoyLog;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>ContactSearchSceneController.java</strong><br>
 * Created: <strong>07.06.2020</strong><br>
 *
 * @author Leon Hofmeister
 * @since Envoy Client v0.1-beta
 */
public class ContactSearchScene {

	@FXML
	private ClearableTextField searchBar;

	@FXML
	private ListView<Chat> chatList;

	private SceneContext sceneContext;

	private LocalDB localDB;

	private static EventBus		eventBus	= EventBus.getInstance();
	private static final Logger	logger		= EnvoyLog.getLogger(ChatScene.class);

	/**
	 * @param sceneContext enables the user to return to the chat scene
	 * @param localDB      the local database to which new contacts are added
	 * @since Envoy Client v0.1-beta
	 */
	public void initializeData(SceneContext sceneContext, LocalDB localDB) {
		this.sceneContext	= sceneContext;
		this.localDB		= localDB;
	}

	@FXML
	private void initialize() {
		chatList.setCellFactory(ContactListCellFactory::new);
		searchBar.setClearButtonListener(e -> { searchBar.getTextField().clear(); chatList.getItems().clear(); });
		eventBus.register(ContactSearchResult.class,
				response -> Platform.runLater(() -> {
					chatList.getItems().clear();
					chatList.getItems().addAll(response.get().stream().map(Chat::new).collect(Collectors.toList()));
				}));
	}

	/**
	 * Disables the clear and search button if no text is present in the search bar.
	 *
	 * @since Envoy Client v0.1-beta
	 */
	@FXML
	private void sendRequest() {
		final var text = searchBar.getTextField().getText().strip();
		if (!text.isBlank()) eventBus.dispatch(new SendEvent(new ContactSearchRequest(text)));
		else chatList.getItems().clear();
	}

	/**
	 * Clears the text in the search bar and the items shown in the list.
	 * Additionally disables both clear and search button.
	 *
	 * @since Envoy Client v0.1-beta
	 */
	@FXML
	private void clear() {
		searchBar.getTextField().setText(null);
		chatList.getItems().clear();
	}

	/**
	 * Sends an {@link ContactOperation} for every selected contact to the
	 * server.
	 *
	 * @since Envoy Client v0.1-beta
	 */
	@FXML
	private void chatListClicked() {
		final var chat = chatList.getSelectionModel().getSelectedItem();
		if (chat != null) {
			final var alert = new Alert(AlertType.CONFIRMATION);
			alert.setTitle("Add Contact to Contact List");
			alert.setHeaderText("Add the user " + chat.getRecipient().getName() + " to your contact list?");
			// Normally, this would be total BS (we are already on the FX Thread), however
			// it could be proven that the creation of this dialog wrapped in
			// Platform.runLater is less error-prone than without it
			Platform.runLater(() -> alert.showAndWait().filter(btn -> btn == ButtonType.OK).ifPresent(btn -> {
				final var event = new ContactOperation(chat.getRecipient(), ElementOperation.ADD);
				// Sends the event to the server
				eventBus.dispatch(new SendEvent(event));
				// Updates the UI
				eventBus.dispatch(event);
				logger.log(Level.INFO, "Added contact " + chat.getRecipient());
			}));
		}
	}

	@FXML
	private void newGroupButtonClicked() {
		sceneContext.load(SceneContext.SceneInfo.GROUP_CREATION_SCENE);
		sceneContext.<GroupCreationScene>getController().initializeData(sceneContext, localDB);
	}

	@FXML
	private void backButtonClicked() { sceneContext.pop(); }
}
